@@ -2,11 +2,17 @@ extern crate bindgen;
 
 fn create_bindings(
     apr_path: &std::path::Path,
+    apu_path: &std::path::Path,
     out_path: &std::path::Path,
     apr_include_paths: &[&std::path::Path],
 ) {
     // Generate bindings using bindgen
-    let bindings = bindgen::Builder::default()
+    let mut builder = bindgen::Builder::default();
+    // check if the pool-debug feature is present
+    if std::env::var("CARGO_FEATURE_POOL_DEBUG").is_ok() {
+        builder = builder.clang_arg("-DAPR_POOL_DEBUG");
+    }
+    let bindings = builder
         .header(apr_path.join("apr.h").to_str().unwrap())
         .header(apr_path.join("apr_allocator.h").to_str().unwrap())
         .header(apr_path.join("apr_general.h").to_str().unwrap())
@@ -18,11 +24,11 @@ fn create_bindings(
         .header(apr_path.join("apr_file_info.h").to_str().unwrap())
         .header(apr_path.join("apr_file_io.h").to_str().unwrap())
         .header(apr_path.join("apr_getopt.h").to_str().unwrap())
-        .header(apr_path.join("apr_uri.h").to_str().unwrap())
+        .header(apu_path.join("apr_uri.h").to_str().unwrap())
         .header(apr_path.join("apr_time.h").to_str().unwrap())
-        .header(apr_path.join("apr_date.h").to_str().unwrap())
+        .header(apu_path.join("apr_date.h").to_str().unwrap())
         .header(apr_path.join("apr_version.h").to_str().unwrap())
-        .header(apr_path.join("apu_version.h").to_str().unwrap())
+        .header(apu_path.join("apu_version.h").to_str().unwrap())
         .header(apr_path.join("apr_strings.h").to_str().unwrap())
         .header(apr_path.join("apr_thread_proc.h").to_str().unwrap())
         .allowlist_file(".*/apr.h")
@@ -61,15 +67,24 @@ fn main() {
 
     let apr = deps.get_by_name("apr-1").unwrap();
 
+    let apr_util = deps.get_by_name("apr-util-1").unwrap();
+
     let apr_path = apr
         .include_paths
         .iter()
         .find(|x| x.join("apr.h").exists())
         .expect("Failed to find apr.h");
 
+    let apr_util_path = apr_util
+        .include_paths
+        .iter()
+        .find(|x| x.join("apu.h").exists())
+        .expect("Failed to find apu.h");
+
     let out_path = std::path::PathBuf::from(std::env::var("OUT_DIR").unwrap());
     create_bindings(
         apr_path.as_path(),
+        apr_util_path.as_path(),
         out_path.as_path(),
         apr.include_paths
             .iter()
