@@ -1,8 +1,27 @@
+//! Memory pool management.
 use crate::generated;
 
+/// A memory pool.
 #[derive(Debug)]
+#[repr(transparent)]
 pub struct Pool(*mut generated::apr_pool_t);
 
+// Pools are Send, but not Sync
+unsafe impl Send for Pool {}
+
+#[cfg(feature = "pool-debug")]
+#[macro_export]
+macro_rules! pool_debug {
+    ($name:ident, $doc:expr) => {
+        #[doc = $doc]
+        pub fn $name(&mut self) -> Self {
+            let mut subpool: *mut generated::apr_pool_t = std::ptr::null_mut();
+            let location = std::concat!(file!(), ":", line!());
+            Pool::new_debug(location)
+        }
+    };
+}
+
 impl Pool {
     /// Create a new pool.
     pub fn new() -> Self {
@@ -12,51 +31,70 @@ impl Pool {
                 &mut pool,
                 std::ptr::null_mut(),
                 None,
-                std::ptr::null_mut() as *mut generated::apr_allocator_t,
+                std::ptr::null_mut(),
             );
         }
         Pool(pool)
     }
 
+    #[cfg(feature = "pool-debug")]
+    pub fn new_debug(location: &str) -> Self {
+        let mut pool: *mut generated::apr_pool_t = std::ptr::null_mut();
+        unsafe {
+            generated::apr_pool_create_ex_debug(
+                &mut pool,
+                std::ptr::null_mut(),
+                None,
+                std::ptr::null_mut(),
+                location.as_ptr() as *const std::ffi::c_char,
+            );
+        }
+        Pool(pool)
+    }
+
+    /// Create a pool from a raw pointer.
     pub fn from_raw(ptr: *mut generated::apr_pool_t) -> Self {
         Pool(ptr)
     }
 
-    pub fn as_ptr(&self) -> *const generated::apr_pool_t {
+    /// Get the raw pointer to the pool.
+    pub unsafe fn as_ptr(&self) -> *const generated::apr_pool_t {
         self.0
     }
 
-    pub fn as_mut_ptr(&mut self) -> *mut generated::apr_pool_t {
+    /// Get the raw mutable pointer to the pool.
+    pub unsafe fn as_mut_ptr(&self) -> *mut generated::apr_pool_t {
         self.0
     }
 
     /// Create a subpool.
-    pub fn subpool(&mut self) -> Self {
+    pub fn subpool(&self) -> Self {
         let mut subpool: *mut generated::apr_pool_t = std::ptr::null_mut();
         unsafe {
-            generated::apr_pool_create_ex(
-                &mut subpool,
-                self.0,
-                None,
-                std::ptr::null_mut() as *mut generated::apr_allocator_t,
-            );
+            generated::apr_pool_create_ex(&mut subpool, self.0, None, std::ptr::null_mut());
         }
         Pool(subpool)
     }
 
+    #[allow(clippy::mut_from_ref)]
     /// Allocate memory in the pool.
-    pub fn alloc<T: Sized>(&mut self) -> *mut std::mem::MaybeUninit<T> {
+    pub fn alloc<T: Sized>(&self) -> &mut std::mem::MaybeUninit<T> {
         let size = std::mem::size_of::<T>();
-        unsafe { generated::apr_palloc(self.0, size) as *mut std::mem::MaybeUninit<T> }
+        unsafe {
+            let x = generated::apr_palloc(self.0, size) as *mut std::mem::MaybeUninit<T>;
+            &mut *x
+        }
     }
 
-    /// Allocate memory in the pool, and initialize it to zero.
-    pub fn calloc<T: Sized>(&mut self) -> *mut T {
-        let data = self.alloc::<T>();
+    /// Allocate memory in the pool and zero it.
+    #[allow(clippy::mut_from_ref)]
+    pub fn calloc<T: Sized>(&self) -> &mut T {
+        let size = std::mem::size_of::<T>();
         unsafe {
-            std::ptr::write_bytes(data as *mut u8, 0, std::mem::size_of::<T>());
+            let x = generated::apr_palloc(self.0, size) as *mut T;
+            std::ptr::write_bytes(x as *mut u8, 0, size);
+            &mut *x
         }
-        data as *mut T
     }
 
     /// Check if the pool is an ancestor of another pool.
@@ -85,11 +123,64 @@ impl Pool {
         }
     }
 
-    /// Get the parent pool, if any.
-    pub fn parent(&self) -> Self {
-        let parent = unsafe { generated::apr_pool_parent_get(self.0) };
-        Pool(parent)
+    /// Clear all memory in the pool, with debug information.
+    ///
+    /// This does not actually free the memory, it just allows the pool to reuse this memory for the next allocation.
+    ///
+    /// # Safety
+    ///
+    /// This is unsafe because it is possible to create a dangling pointer to memory that has been cleared.
+    #[cfg(feature = "pool-debug")]
+    pub unsafe fn fn_clear_debug(&mut self, location: &str) {
+        unsafe {
+            generated::apr_pool_clear_debug(self.0, location.as_ptr() as *const std::ffi::c_char);
+        }
     }
+
+    /// Get the parent pool, if any.
+    pub fn parent<'a, 'b>(&'a self) -> Option<&'b Self>
+    where
+        'b: 'a,
+    {
+        let parent = unsafe { generated::apr_pool_parent_get(self.0) };
+        if parent.is_null() {
+            None
+        } else {
+            Some(unsafe { &*(parent as *const Pool) })
+        }
+    }
+
+    /// Run all registered child cleanups, in preparation for an exec() call.
+    pub fn cleanup_for_exec(&self) {
+        unsafe {
+            generated::apr_pool_cleanup_for_exec();
+        }
+    }
+
+    /// Try to join two pools.
+    #[cfg(feature = "pool-debug")]
+    pub fn join(&self, other: &Pool) {
+        unsafe { generated::apr_pool_join(self.0, other.0) }
+    }
+
+    #[cfg(feature = "pool-debug")]
+    pub fn num_bytes(&self, recurse: bool) -> usize {
+        unsafe { generated::apr_pool_num_bytes(self.0, if recurse { 1 } else { 0 }) }
+    }
+
+    #[cfg(feature = "pool-debug")]
+    pub unsafe fn find(&self, ptr: *const std::ffi::c_void) -> Option<Pool> {
+        let pool = generated::apr_pool_find(ptr);
+        if pool.is_null() {
+            None
+        } else {
+            Some(Pool(pool))
+        }
+    }
+
+    /// Try to join two pools.
+    #[cfg(not(feature = "pool-debug"))]
+    pub fn join(&self, _other: &Pool) {}
 }
 
 impl Default for Pool {
@@ -105,9 +196,12 @@ impl Drop for Pool {
         }
     }
 }
+
+/// An allocator.
 pub struct Allocator(*mut generated::apr_allocator_t);
 
 impl Allocator {
+    /// Create a new allocator.
     pub fn new() -> Self {
         let mut allocator: *mut generated::apr_allocator_t = std::ptr::null_mut();
         unsafe {
@@ -116,6 +210,7 @@ impl Allocator {
         Allocator(allocator)
     }
 
+    /// Return the raw pointer to the allocator.
     pub fn as_ptr(&self) -> *const generated::apr_allocator_t {
         self.0
     }
@@ -141,11 +236,14 @@ mod tests {
 
     #[test]
     fn test_pool() {
-        let mut pool = Pool::new();
+        let pool = Pool::new();
+        assert!(pool.parent().unwrap().is_ancestor(&pool));
+        let parent = pool.parent();
+        assert!(parent.unwrap().is_ancestor(&pool));
         let subpool = pool.subpool();
         assert!(pool.is_ancestor(&subpool));
         assert!(!subpool.is_ancestor(&pool));
-        assert!(subpool.parent().is_ancestor(&subpool));
+        assert!(subpool.parent().unwrap().is_ancestor(&subpool));
         subpool.tag("subpool");
         pool.tag("pool");
     }
@@ -154,6 +252,7 @@ mod tests {
 /// A wrapper around a value that is allocated in a pool.
 pub struct Pooled<T> {
     pool: std::rc::Rc<Pool>,
+    /// The value.
     pub data: T,
 }
 
@@ -238,6 +337,7 @@ impl<T> PooledPtr<T> {
         PooledPtr { pool, data }
     }
 
+    /// Check if the pointer is null.
     pub fn is_null(&self) -> bool {
         self.data.is_null()
     }
@@ -284,6 +384,17 @@ impl<T> std::fmt::Debug for PooledPtr<T> {
     }
 }
 
+/// Terminate the apr pool subsystem.
+///
+/// # Safety
+///
+/// This function is unsafe because it is possible to create a dangling pointer to memory that has been cleared.
+pub unsafe fn terminate() {
+    unsafe {
+        generated::apr_pool_terminate();
+    }
+}
+
 #[cfg(test)]
 mod pooled_tests {
     #[test]
